<?php

namespace App\Http\Controllers;

use App\Audio;
use App\AudioLanguage;
use App\Genre;
use App\Audiogenre;
use App\Menu;
use App\WorldCupTeam;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Intervention\Image\Facades\Image;

class WorldCupTeamController extends Controller
{

    public function index(Request $request)
    {
        
        if($request->search != NULL){
            $teams = DB::table('world_cup_teams')->where('name', 'like','%' . $request->search . '%')->paginate(12);
        }else{
            $teams = DB::table('world_cup_teams')->paginate(12);
        }
        
        return view('admin.worldcupteam.index', compact('teams'));
    }

    public function create()
    {
        $genre_ls = Genre::pluck('name', 'id')->all();
        return view('admin.worldcupteam.create', compact( 'genre_ls'));
    }

    public function destroy($id)
    {
        if(env('DEMO_LOCK') == 1){
            return back()->with('deleted','This action is disabled in the demo !');
        }
        $new = WorldCupTeam::findOrFail($id);

        if ($new->image != null) {
            $content = @file_get_contents(public_path() . '/images/worldcupteam/' . $new->image);
            if ($content) {
                unlink(public_path() . "/images//worldcupteam/" . $new->image);
            }
        }

        $new->delete();
        return back()->with('deleted', 'Team has been deleted');

    }

    public function edit($id)
    {
        $team = WorldCupTeam::findOrFail($id);
        $genre_ls = Genre::pluck('name', 'id')->all();
     
        return view('admin.worldcupteam.edit', compact('team' ,'genre_ls'));

    }

    public function update(Request $request, $id){
        $team = WorldCupTeam::findOrFail($id);
        $input = $request->all();

        if ($file = $request->file('image')) {
          
            $validator = Validator::make(
                [
                    'image' => $request->image,
                    'extension' => strtolower($request->image->getClientOriginalExtension()),
                ],
                [
                    'image' => 'required',
                    'extension' => 'required|in:jpg,jpeg,png,webp',
                ]
            );
            if ($validator->fails()) {
                return back()->with('deleted','Invalid file format Please use jpg,webp,jpeg and png image format !')->withInput();
            }else{
                $image = "team_" . time() . $file->getClientOriginalName();
                if ($team->image != null) {
                    $content = @file_get_contents(public_path() . '/images/worldcupteam/' . $team->image);
                    if ($content) {
                        unlink(public_path() . "/images/worldcupteam/" . $team->image);
                    }
                   
                      
                }
                $img = Image::make($file->path());

                $img->resize(50, 50, function ($constraint) {
                    $constraint->aspectRatio();
                });
                $img->save(public_path('/images/worldcupteam') .'/'. $image);
         
                  
                 $input['image'] = $image;
                
             }
          
        }

        $team->update($input);
        return redirect('/admin/worldcupteam')->with('updated', 'Team has been updated');
    }

    public function store(Request $request)
    {
        
        $request->validate(['name' => 'required' , 'genre_id' =>'required']);

        if ($file = $request->file('image')) {

            $validator = Validator::make(
                [
                    'image' => $request->image,
                    'extension' => strtolower($request->image->getClientOriginalExtension()),
                ],
                [
                    'image' => 'required',
                    'extension' => 'required|in:jpg,jpeg,png,webp',
                ]
            );
            if ($validator->fails()) {
                return back()->with('deleted','Invalid file format Please use jpg,webp,jpeg and png image format !')->withInput();
            }else{

                $image = "team_" . time() . $file->getClientOriginalName();
                if ($request->image != null) {
                    $content = @file_get_contents(public_path() . '/images/worldcupteam/' . $request->image);
                    if ($content) {
                        unlink(public_path() . "/images/worldcupteam/" . $request->image);
                    }
                }
                $img = Image::make($file->path());

                $img->resize(50, 50, function ($constraint) {
                    $constraint->aspectRatio();
                });
                $img->save(public_path('/images/worldcupteam') .'/'. $image);
               
                $input['image'] = $image;
            }
        }

        $input = $request->all();
        $team = WorldCupTeam::create($input);
        return back()->with('added', 'Audio has been added');

    }

}